﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace GameOfLife
{
    /// <summary>
    /// Base class for boards which store their Data in a single byte array,
    /// organised in the natural way.
    /// </summary>
    abstract class BytePerPixelBoard : IBoard
    {
        private Bitmap bitmap;
        protected byte[] Data { get; private set; }
        protected int Width { get; private set; }
        protected int Height { get; private set; }

        protected BytePerPixelBoard(int width, int height)
        {
            this.Width = width;
            this.Height = height;
            Data = new byte[width * height];
            CreateInitialImage();
        }

        void CreateInitialImage()
        {
            bitmap = new Bitmap(Width, Height, PixelFormat.Format8bppIndexed);
            ColorPalette palette = bitmap.Palette;
            palette.Entries[0] = Color.Black;
            palette.Entries[1] = Color.White;
            bitmap.Palette = palette;
        }

        public Image Render()
        {
            Rectangle rect = new Rectangle(0, 0, Width, Height);
            BitmapData bmpData = bitmap.LockBits(rect, ImageLockMode.ReadWrite, bitmap.PixelFormat);
            Marshal.Copy(Data, 0, bmpData.Scan0, Data.Length);
            bitmap.UnlockBits(bmpData);

            return bitmap;
        }

        public bool this[int x, int y]
        {
            get
            {
                return Data[y * Width + x] == 1;
            }
            set
            {
                Data[y * Width + x] = value ? (byte)1 : (byte)0;
            }
        }

        public abstract void CreateNextGeneration(IBoard parent);
    }
}
