﻿using System.Threading;
namespace GameOfLife
{
    /// <summary>
    /// Like SimpleFastInteriorBoard, but using a parellel for loop.
    /// </summary>
    class ParallelFastInteriorBoard : BytePerPixelBoard
    {
        int[] offsets;

        public ParallelFastInteriorBoard(int width, int height)
            : base(width, height)            
        {
            offsets = new int[] { -width-1, -width, -width+1, -1, 1, width - 1, width, width + 1 };
        }        

        public override void CreateNextGeneration(IBoard parent)
        {
            ParallelFastInteriorBoard parentBoard = (ParallelFastInteriorBoard)parent;
            byte[] pd = parentBoard.Data;

            Parallel.For(0, Height, y => 
            {
                bool topOrBottom = y == 0 || y == Height - 1;
                int index = y * Width;
                for (int x = 0; x < Width; x++)
                {
                    bool alive = parentBoard[x, y];

                    int neighbours=0;
                    if (topOrBottom || x == 0 || x == Width - 1)
                    {
                        for (int dx = -1; dx < 2; dx++)
                        {
                            for (int dy = -1; dy < 2; dy++)
                            {
                                // Ignore ourselves
                                if (dx == 0 && dy == 0)
                                {
                                    continue;
                                }
                                int neighbourIndex = ((y + dy + Height) % Height) * Width + (x + dx + Width) % Width;
                                neighbours += pd[neighbourIndex];
                            }
                        }
                    }
                    else
                    {
                        for (int i = 0; i < offsets.Length; i++)
                        {
                            neighbours += pd[index + offsets[i]];
                        }
                    }
                    this[x, y] = neighbours == 3 || (alive && neighbours == 2);
                    index++;
                }
            });
        }
    }
}
