﻿namespace GameOfLife
{
    /// <summary>
    /// Board which calculates its generations in a completely unoptimized way.
    /// </summary>
    class SimpleBoard : BytePerPixelBoard
    {
        public SimpleBoard(int width, int height) : base(width, height)            
        {
        }        

        public override void CreateNextGeneration(IBoard parent)
        {
            SimpleBoard parentBoard = (SimpleBoard)parent;
            byte[] pd = parentBoard.Data;

            int index = 0;

            for (int y = 0; y < Height; y++)
            {
                for (int x = 0; x < Width; x++)
                {
                    bool alive = parentBoard[x, y];

                    int neighbours=0;
                    for (int dx = -1; dx < 2; dx++)
                    {
                        for (int dy = -1; dy < 2; dy++)
                        {
                            // Ignore ourselves
                            if (dx == 0 && dy == 0)
                            {
                                continue;
                            }
                            int neighbourIndex = ((y + dy + Height) % Height) * Width + (x + dx + Width) % Width;
                            neighbours += pd[neighbourIndex];
                        }
                    }
                    this[x, y] = neighbours == 3 || (alive && neighbours == 2);
                    index++;
                }
            }
        }
    }
}
