﻿namespace GameOfLife
{
    /// <summary>
    /// Each pixel is still calculated, but we use offsets for fast calculation of the interior;
    /// edges are still calculated in a naive manner.
    /// </summary>
    class SimpleFastInteriorBoard : BytePerPixelBoard
    {
        int[] offsets;

        public SimpleFastInteriorBoard(int width, int height)
            : base(width, height)            
        {
            offsets = new int[] { -width-1, -width, -width+1, -1, 1, width - 1, width, width + 1 };
        }        

        public override void CreateNextGeneration(IBoard parent)
        {
            SimpleFastInteriorBoard parentBoard = (SimpleFastInteriorBoard)parent;
            byte[] pd = parentBoard.Data;

            int index = 0;

            for (int y = 0; y < Height; y++)
            {
                bool topOrBottom = y == 0 || y == Height - 1;
                for (int x = 0; x < Width; x++)
                {
                    bool alive = parentBoard[x, y];

                    int neighbours=0;
                    if (topOrBottom || x == 0 || x == Width - 1)
                    {
                        for (int dx = -1; dx < 2; dx++)
                        {
                            for (int dy = -1; dy < 2; dy++)
                            {
                                // Ignore ourselves
                                if (dx == 0 && dy == 0)
                                {
                                    continue;
                                }
                                int neighbourIndex = ((y + dy + Height) % Height) * Width + (x + dx + Width) % Width;
                                neighbours += pd[neighbourIndex];
                            }
                        }
                    }
                    else
                    {
                        for (int i = 0; i < offsets.Length; i++)
                        {
                            neighbours += pd[index + offsets[i]];
                        }
                    }
                    this[x, y] = neighbours == 3 || (alive && neighbours == 2);
                    index++;
                }
            }
        }
    }
}
